/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.commons.threads;

import org.almendra.commons.util.AssertUtility;

/**
 * @author Oliver
 *
 */
public class CountDown extends Future
{
	int count = -1;
	
	/**
	 * Creates a new <code>CountDown</code> object.
	 */
	private CountDown()
	{
		super();
	}
	
	/**
	 * Creates a new <code>CountDown</code> object.
	 */
	public CountDown(int count)
	{
		super();
		this.count = count;
		AssertUtility.assertTrue("count must be larger than 0!", count > 0); //$NON-NLS-1$
	}
	
	public void decrement()
	{
	    accessProtection.lock();
	    try
	    {
	    	count--;
			if(count == 0)
			{
				super.signal();
			}
		}
	    finally
	    {
	        accessProtection.unlock();
	    }
	}

    /* (non-Javadoc)
     * @see org.almendra.commons.threads.ISignalable#signal()
     */
    public void signal()
    {
        decrement();
    }
}
