/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.commons.util.collection;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author Oliver
 */
public class ComparableValuesHashMap<K, V extends Comparable> extends HashMap<K, V>
{

    private static final long serialVersionUID = -3303351323169228008L;

	/**
	 * @param initialCapacity
	 * @param loadFactor
	 */
	public ComparableValuesHashMap(int initialCapacity, float loadFactor) 
	{
		super(initialCapacity, loadFactor);
	}

	/**
	 * @param initialCapacity
	 */
	public ComparableValuesHashMap(int initialCapacity) 
	{
		super(initialCapacity);
	}

	/**
	 * 
	 */
	public ComparableValuesHashMap() 
	{
		super();
	}

	/**
	 * @param m
	 */
	public ComparableValuesHashMap(Map<K, V> m) 
	{
		super(m);
	}
	
	public List<Integer> getValuesInOrder(List<K> keysInOrder) throws IllegalArgumentException
	{
		List<Integer> values = new LinkedList<Integer>();
		for(K key : keysInOrder)
		{
		    // TODO - find out why this does not compile without cast although the type is clearly specified
			Integer value = (Integer) get(key);
			if(value == null)
				throw new IllegalArgumentException("value for " + key + " missing in map");  //$NON-NLS-1$ //$NON-NLS-2$
			values.add(value);
		}
		return values;
	}
	
	public List<K> getKeysWithGreatestIntegerValue(ComparableValuesHashMap<K, V> map)
	{		
		List<K> keysOut = new ArrayList<K>();
		if(map.isEmpty())
			return keysOut;
		List<V> values = ListUtility.toList(map.values());
		Collections.sort(values);
		V largest = values.get(values.size() - 1);
		Set<K> keys = map.keySet();
		for(K key : keys)
		{
			V value = (V) map.get(key);
			if(value.equals(largest))
				keysOut.add(key);
		}
		return keysOut;
	}
		

}
