/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project (http://www.almendra.org/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.commons.util.collection;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.almendra.commons.util.RandomUtility;


 /** 
  * Defines some utility methods to deal with lists not found 
  * in other classes of the Java collection framework.
  * 
  * @author Oliver Plohmann
  */

public class ListUtility
{

	/**
	 * Creates a new <code>ListUtility</code> object. Not used since all methods are static.
	 */
	protected ListUtility()
	{
		super();
	}

	/**
	 * Create a list from two given objects.
	 * @param object1
	 * @param object2
	 * @return
	 */
	public static <T> List<T> create(T object1, T object2)
	{
		List<T> list = new ArrayList<T>();
		list.add(object1);
		list.add(object2);
		return list;
	}

	/**
	 * Create a list from three given objects.
	 * @param object1
	 * @param object2
	 * @param object3
	 * @return
	 */
	public static <T> List<T> create(T object1, T object2, T object3)
	{
		List<T> list = new ArrayList<T>();
		list.add(object1);
		list.add(object2);
		list.add(object3);
		return list;
	}

	/**
	 * Create a list from four given objects.
	 * @param object1
	 * @param object2
	 * @param object3
	 * @param object4
	 * @return
	 */
	public static <T> List<T> create(T t1, T t2, T t3, T t4)
	{
		List<T> list = new ArrayList<T>();
		list.add(t1);
		list.add(t2);
		list.add(t3);
		list.add(t4);
		return list;
	}

	/**
	 * Turn a collection into a list. Returns a <code>List</code> from which you can remove
	 * elements contrary to Arrays.asList(...) which returns an ArrayList.
	 * @param set to be converted
	 * @return
	 */
	public static <T> List<T> toList(Collection<T> c)
	{
		List<T> list = new ArrayList<T>(c.size());
		for(T t : c)
			list.add(t);
		return list;
	}
	
	public static <T> List<T> toList(T c[])
	{
		List<T> list = new ArrayList<T>(c.length);
		for(T t : c)
			list.add(t);
		return list;
	}
	
	/**
	 * @param entry
	 * @return
	 */
	public static <T> List<T> toListFromSingleObject(T t)
	{
		List<T> list = new ArrayList<T>();
		list.add(t);
		return list;
	}
	
	/**
	 * @param entry
	 * @return
	 */
	public static List<String> toStringListFromSingleObject(String str)
	{
		List<String> list = new StringList();
		list.add(str);
		return list;
	}

	/**
	 * @param localCapabilityPaths1
	 * @param localCapabilityPaths2
	 * @return
	 */
	public static <T> List<T> getIntersection(Collection<T> coll1, List<T> coll2)
	{
		List<T> intersection = new ArrayList<T>();
		Collection<T> smallerColl = null;
		Collection<T> largerColl = null;
		if(coll1.size() >= coll2.size())
		{
			smallerColl = coll2;
			largerColl = coll1;
		}
		else
		{
			smallerColl = coll1;
			largerColl = coll2;
		}
		for(T t : smallerColl)
		{
			if(largerColl.contains(t))
				intersection.add(t);
		}
		return intersection;
	}
	
	public static <T> T pickAtRandom(List<T> list)
	{
		return list.get(RandomUtility.Instance.nextInt(list.size()));
	}
	
	public static List<String> toStringList(Set<String> strings)
	{
		List<String> list = new StringList();
		for(String str : strings)
			list.add(str);
		return list;
	}

	/**
	 * @param agentNames
	 * @return
	 */
	public static List<String> toStringList(String[] strings) 
	{
		List<String> stringList = new StringList();
		for(String str : strings)
			stringList.add(str);
		return stringList;
	}

	/**
	 * @param list
	 * @return
	 */
	public static List<String> toStringList(List<String> list) {
		List<String> strings = new StringList();
		for(String str : list)
			strings.add(str);
		return null;
	}
}
