/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.commons.util.collection;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.almendra.commons.util.RandomUtility;



public class MapUtility
{
	/**
	 * Creates a new <code>MapUtility</code> object.
	 */
	public MapUtility()
	{
		super();
	}
	
	public static <K, V> K keyOfValue(Map<K, V> map, V value)
	{
	    if(value == null)
	        return null;
	    for(K key : map.keySet())
	    {
	        V currentValue = map.get(key);
	        if(value.equals(currentValue))
	            return key;
	    }
		return null;
	}
	
	public static <K, V> List<V> getValuesInOrder(Map<K, V> map, List<K> keysInOrder) throws IllegalArgumentException
	{
		List<V> values = new LinkedList<V>();
		for(K k : keysInOrder)
		{
			V value = map.get(k);
			if(value == null)
				throw new IllegalArgumentException("value for " + k + " missing in map");  //$NON-NLS-1$ //$NON-NLS-2$
			values.add(value);
		}
		return values;
	}
	
	public static <K> List<K> getKeysWithSmallestIntegerValue(Map<K, Integer> map)
	{		
		List<K> keys = new ArrayList<K>();
		if(map.isEmpty())
			return keys;
		List<Integer> values = ListUtility.toList(map.values());
		Collections.sort(values);
		int smallest = values.get(0).intValue();
		for(K key : map.keySet())
		{
			int value = map.get(key).intValue();
			if(value == smallest)
				keys.add(key);
		}
		return keys;
	}
	
	public static <K> K getKeyWithGreatestIntegerValue(Map<K, Integer> map)
	{
		List<K> keys = getKeysWithGreatestIntegerValue(map);
		if(keys.isEmpty())
			return null;
		return keys.get(RandomUtility.Instance.nextInt(keys.size()));
	}
	
	public static <K> List<K> getKeysWithGreatestIntegerValue(Map<K, Integer> map)
	{		
		List<K> keys = new ArrayList<K>();
		if(map.isEmpty())
			return keys;
		List<Integer> values = ListUtility.toList(map.values());
		Collections.sort(values);
		int largest = values.get(values.size() - 1).intValue();
		for(K key : map.keySet())
		{
			int value = map.get(key).intValue();
			if(value == largest)
				keys.add(key);
		}
		return keys;
	}
	
	public static <K> List<K> getKeysWithSmallestLongValue(Map<K, Long> map)
	{		
		List<K> keys = new ArrayList<K>();
		if(map.isEmpty())
			return keys;
		List<Long> values = ListUtility.toList(map.values());
		Collections.sort(values);
		int smallest = values.get(0).intValue();
		for(K key : map.keySet())
		{
			int value = map.get(key).intValue();
			if(value == smallest)
				keys.add(key);
		}
		return keys;
	}

	/**
	 * @param queueSizeByAgentPath
	 * @return
	 */
	public static <K> K getKeyWithSmallestIntegerValue(Map<K, Integer> queueSizeByAgentPath)
	{		
		List<K> keys = getKeysWithSmallestIntegerValue(queueSizeByAgentPath);
		if(keys.isEmpty())
			return null;
		return keys.get(RandomUtility.Instance.nextInt(keys.size()));
	}

	/**
	 * @param capabilitiesByCandidateNode
	 * @param string
	 * @param nodes
	 */
	public static <K, V> void putAll(Map<K, List<V>> map, V value, Collection<K> keys)
	{
		for(K key : keys)
		{
			List<V> values = map.get(key);
			if(values == null)
			{
				values = new ArrayList<V>();
				map.put(key, values);
			}
			values.add(value);
		}
	}
	
	/**
	 * @param capabilitiesByCandidateNode
	 * @param string
	 * @param nodes
	 */
	public static <K> void putAllStrings(Map<K, List<String>> map, String value, Collection<K> keys)
	{
		for(K key : keys)
		{
			List<String> values = map.get(key);
			if(values == null)
			{
				values = new StringList();
				map.put(key, values);
			}
			values.add(value);
		}
	}

	/**
	 * @param capabilitiesByCandidateNodeName
	 * @param qualifiedPathName
	 * @param nodeNames
	 */
	public static void putAllStrings(Map capabilitiesByCandidateNodeName, String qualifiedPathName, Set nodeNames) {
		// TODO Auto-generated method stub
		
	}
	
}
