package org.almendra.commons.util.swing;

import javax.swing.JTextArea;
import javax.swing.text.BadLocationException;

import org.almendra.commons.number.NumberUtility;
import org.almendra.commons.util.AssertUtility;
import org.almendra.commons.util.StringUtility;
import org.almendra.commons.util.collection.StringList;
import org.almendra.commons.util.logging.ErrorLogger;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

/**
 * Insert the type's description here. Creation date: (08.03.2004 13:22:52)
 * @author: Administrator
 */
public class BillboardPanelController
{
	public static final String LABEL = "BillboardPanelController"; //$NON-NLS-1$
	public static final String BLANK_LINE = "BLANK_LINE"; //$NON-NLS-1$
	protected String separator = "="; //$NON-NLS-1$
	protected List<String> masksOrder = new StringList();
	protected List<String> hiddenMasks = new StringList();
	protected Map<String, List<String>> masks = new java.util.HashMap<String, List<String>>();
	protected Map<String, String> valuesByAttribute = new java.util.HashMap<String, String>();
	protected JTextArea textArea = null;
	protected Lock accessProtection = new ReentrantLock();

	/**
	 * NodeViewController constructor comment.
	 */
	protected BillboardPanelController()
	{
		super();
	}

	/**
	 * NodeViewController constructor comment.
	 */
	public BillboardPanelController(JTextArea textArea)
	{
		super();
		this.textArea = textArea;
	}

	/**
	 * NodeViewController constructor comment.
	 */
	public BillboardPanelController(JTextArea textArea, String separator)
	{
		super();
		this.textArea = textArea;
		this.separator = separator;
	}

	/**
	 * NodeViewController constructor comment.
	 */
	protected void fillInitialMask(List<String> attributeMask)
	{
		for(String attribute :  attributeMask)
			setAttributeValuePair(attribute, "", textArea.getLineCount()); //$NON-NLS-1$
	}

	protected void setAttributeValuePair(String attribute, String value, int line)
	{
		if (BLANK_LINE.equals(attribute))
		{
			textArea.append("\n"); //$NON-NLS-1$
			return;
		}
		valuesByAttribute.put(attribute, value);
		String attributeValuePair = "    " + StringUtility.rightPadded(attribute, 42) + " " + separator + "    " + value + "\n"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		int lineCount = textArea.getLineCount();
		if (lineCount <= line)
		{
			for (int i = lineCount; i < line; i++)
			{
				textArea.append("\n"); //$NON-NLS-1$
			}
			textArea.append(attributeValuePair); //$NON-NLS-1$
			return;
		}
		try
		{
			int start = textArea.getLineStartOffset(line);
			int end = textArea.getLineEndOffset(line);
			textArea.replaceRange(attributeValuePair, start, end);
		}
		catch (BadLocationException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			ErrorLogger.exception(e);
		}
	}

	protected Integer getNextFreeItemNumber()
	{
		int itemNumber = 0;
		for(List<String> attributeMask : masks.values())
			itemNumber += attributeMask.size();
		return new Integer(itemNumber);
	}

	/**
	 * NodeViewController constructor comment.
	 */
	public void insertAttribute(String maskName, String newAttribute, String afterAttributeOrNull, String value)
	{
	    accessProtection.lock();
		try
		{			
			if (!masks.containsKey(maskName))
				return;
			List<String> attributeMask = masks.get(maskName);
			if (attributeMask.contains(newAttribute))
				return;
			String afterAttribute = afterAttributeOrNull;
			if (afterAttribute == null || !attributeMask.contains(afterAttribute))
			{
				afterAttribute = attributeMask.get(attributeMask.size() - 1);
				attributeMask.add(newAttribute);
			}
			else
			{
				attributeMask.add(attributeMask.indexOf(afterAttribute) + 1, newAttribute);
			}
			int lastLinePos = getAttributePos(maskName, afterAttribute) + 1;
			textArea.insert("", lastLinePos); //$NON-NLS-1$
			setAttributeValuePair(newAttribute, value, lastLinePos);
		}
		finally
		{
			accessProtection.unlock();
		}
	}

	/**
	 * NodeViewController constructor comment.
	 */
	public void setMask(String maskName, List<String> attributeMask)
	{
	    accessProtection.lock();
		try
		{			
			if (masks.containsKey(maskName))
				return;
			masks.put(maskName, attributeMask);
			masksOrder.add(maskName);
			fillInitialMask(attributeMask);
		}
		finally
		{
			accessProtection.unlock();
		}
	}

	public void incrementValue(String maskName, String attributeName)
	{
		changeValue(maskName, attributeName, true);
	}

	public void decrementValue(String maskName, String attributeName)
	{
		changeValue(maskName, attributeName, false);
	}

	public String getValue(String maskName, String attributeName)
	{
	    accessProtection.lock();
		try
		{			
			if(hiddenMasks.contains(maskName))
				return null;
			int attributePos = getAttributePos(maskName, attributeName);
			if (attributePos == -1)
				return null;
			String element = ""; //$NON-NLS-1$
			try
			{
				int start = textArea.getLineStartOffset(attributePos);
				int end = textArea.getLineEndOffset(attributePos);
				element = textArea.getText(start, end - start);
			}
			catch (BadLocationException e)
			{
				System.out.println(e.getMessage());
				e.printStackTrace();
				ErrorLogger.exception(e);
				return null;
			}
			int index = element.indexOf(separator);
			AssertUtility.assertTrue(index != 1);
			return element.substring(index + 1, element.length()).trim();
		}
		finally
		{
			accessProtection.unlock();
		}
	}
	
	protected void changeValue(String maskName, String attributeName, boolean increment)
	{
	    accessProtection.lock();
		try
		{			
			if(hiddenMasks.contains(maskName))
				return;
			int attributePos = getAttributePos(maskName, attributeName);
			if (attributePos == -1)
				return;
			// TODO - check whether element really contains line
			String element = ""; //$NON-NLS-1$
			try
			{
				int start = textArea.getLineStartOffset(attributePos);
				int end = textArea.getLineEndOffset(attributePos);
				element = textArea.getText(start, end - start);
			}
			catch (BadLocationException e)
			{
				System.out.println(e.getMessage());
				e.printStackTrace();
				ErrorLogger.exception(e);
				return;
			}
			int index = element.indexOf(separator);
			AssertUtility.assertTrue(index != 1);
			String value = element.substring(index + 1, element.length()).trim();
			try
			{
				int number = Integer.parseInt(value);
				if (increment)
					number++;
				else
					number--;
				setValue(maskName, attributeName, String.valueOf(number));
			}
			catch (NumberFormatException e)
			{
				System.out.println("not a number: " + value); //$NON-NLS-1$
				System.out.println(e.getMessage());
				e.printStackTrace();
				ErrorLogger.exception(e);
				return;
			}
		}
		finally
		{
			accessProtection.unlock();
		}
	}

	protected int getAttributePos(String maskName, String attributeName)
	{
		int nodePos = getMaskStartPosition(maskName);
		if (nodePos == -1)
			return -1;
		int internalAttributePos = masks.get(maskName).indexOf(attributeName);
		return nodePos + internalAttributePos;
	}

	protected int getMaskStartPosition(String maskName)
	{
		if (!masksOrder.contains(maskName))
			return -1;
		int startPos = 0;
		for(String currentMaskName : masksOrder)
		{
			if (currentMaskName.equals(maskName))
				return startPos;
			List<String> mask = masks.get(currentMaskName);
			startPos += mask.size();
		}
		return startPos;
	}

	/**
	 * NodeViewController constructor comment.
	 */
	public void setValue(String maskName, String attributeName, String value)
	{
	    accessProtection.lock();
		try
		{			
			if(hiddenMasks.contains(maskName))
				return;
			int attributePos = getAttributePos(maskName, attributeName);
			if (attributePos == -1)
				return;
			int lineCount = textArea.getLineCount();
			for (int i = lineCount; i <= attributePos; i++)
			{
				textArea.append(""); //$NON-NLS-1$
			}
			setAttributeValuePair(attributeName, value, attributePos);
		}
		finally
		{
			accessProtection.unlock();
		}
	}

	/**
	 * 
	 */
	public void reset()
	{
	    accessProtection.lock();
		try
		{			
			textArea.setText(""); //$NON-NLS-1$
			masks = new HashMap<String, List<String>>();
		}
		finally
		{
			accessProtection.unlock();
		}
	}

	/**
	 * @param string
	 */
	public void unsetMask(String maskName)
	{
	    accessProtection.lock();
		try
		{			
			if(hiddenMasks.contains(maskName))
				return;
			int nodeStartPos = getMaskStartPosition(maskName);
			if (nodeStartPos == -1)
				return;
			List<String> mask = masks.get(maskName);
			for (int i = 0; i < mask.size(); i++)
			{
				if (textArea.getLineCount() > nodeStartPos)
				{
					try
					{
						int start = textArea.getLineStartOffset(nodeStartPos);
						int end = textArea.getLineEndOffset(nodeStartPos);
						textArea.replaceRange("", start, end); //$NON-NLS-1$
					}
					catch (BadLocationException e)
					{
						System.out.println(e.getMessage());
						e.printStackTrace();
						ErrorLogger.exception(e);
						return;
					}
				}
			}
			masks.remove(maskName);
			masksOrder.remove(maskName);
		}
		finally
		{
			accessProtection.unlock();
		}
	}

	/**
	 * 
	 */
	public void resetCounts()
	{
	    accessProtection.lock();
		try
		{			
			for(String maskName : masks.keySet())
			{
				List<String> values = masks.get(maskName);
			    for(String attributeName : values)
				{
					String value = getValue(maskName, attributeName);
					if(NumberUtility.isNumeric(value))
						setValue(maskName, attributeName, "0"); //$NON-NLS-1$
				}    
			}
		}
		finally
		{
			accessProtection.unlock();
		}
	}
	
	public List<String> getMaskNames()
	{
	    accessProtection.lock();
		try
		{			
			List<String> maskNames = new StringList();
			for(String maskName : masks.keySet())
				maskNames.add(maskName);
			return maskNames;
		}
		finally
		{
			accessProtection.unlock();
		}
	}

	/**
	 * @param list
	 */
	public void hideMasks(List<String> maskNames)
	{
	    accessProtection.lock();
		try
		{			
			for(String maskName : maskNames)
			{
				unsetMask(maskName);
				hiddenMasks.add(maskName);
			}
		}
		finally
		{
			accessProtection.unlock();
		}
	}
}
