/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.ade.commands;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import org.almendra.commons.number.Long;
import org.almendra.janet.cas.scheduling.ICommand;



public class NotifyQSCChangedCommand implements ICommand, Serializable
{
	private static final long serialVersionUID = -3272640461649269821L;

	public static final String QualifiedName = "org.almendra.commons.janet.ade.commands.NotifyQSCChangedCommand"; //$NON-NLS-1$
	
	protected String nodeName = null;
	protected int newQueueSizeCategory = -1;
	protected long waitingTime = -1;
	protected Map<String, Long> waitingQueuesByCapability = new HashMap<String, Long>();
	protected Map<String, Long> executingQueuesByCapability = new HashMap<String, Long>();
	
	/**
	 * Creates a new <code>NotifyQSCChangedCommand</code> object.
	 */
	protected NotifyQSCChangedCommand()
	{
		super();
	}
	
	/**
	 * Creates a new <code>NotifyQSCChangedCommand</code> object.
	 */
	public NotifyQSCChangedCommand(String nodeName, int newQueueSizeCategory, long waitingTime, Map<String, Long> waitingQueuesByCapability, Map<String, Long> executingQueuesByCapability)
	{
		super();
		this.nodeName = nodeName;
		this.newQueueSizeCategory = newQueueSizeCategory;
		this.waitingTime = waitingTime;
		this.waitingQueuesByCapability = waitingQueuesByCapability;
		this.executingQueuesByCapability = executingQueuesByCapability;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.ICommand#getQualifiedName()
	 */
	public String getQualifiedName()
	{
		return QualifiedName;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.ICommand#getPriority()
	 */
	public int getPriority()
	{
		return ICommand.ApplicationMaxPriority;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.ICommand#toLogString()
	 */
	public String toLogString()
	{
		return "NotifyQSCChangedCommand{nodeName=" + nodeName + " | newQueueSizeCategory=" + newQueueSizeCategory + "}"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}
	

	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	public Object clone() throws CloneNotSupportedException
	{
		NotifyQSCChangedCommand cmd = (NotifyQSCChangedCommand) super.clone();
		return cmd;
	}
	
	/**
	 * @return Returns the newQueueSizeCategory.
	 */
	public int getNewQueueSizeCategory()
	{
		return newQueueSizeCategory;
	}
	/**
	 * @return Returns the nodeName.
	 */
	public String getNodeName()
	{
		return nodeName;
	}
	/**
	 * @return Returns the executingCommandsCountByCapability.
	 */
	public Map<String, Long> getExecutingQueuesByCapability()
	{
		return executingQueuesByCapability;
	}
	/**
	 * @return Returns the waitingCommandsCountByCapability.
	 */
	public Map<String, Long> getWaitingQueuesByCapability()
	{
		return waitingQueuesByCapability;
	}
	/**
	 * @return Returns the waitingTime.
	 */
	public long getWaitingTime()
	{
		return waitingTime;
	}
}
