/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.ade.distributor;

import java.util.Set;

import org.almendra.commons.Result;
import org.almendra.commons.events.AbstractVetoableCallbackHandler;
import org.almendra.commons.events.Event;


/**
 * @author Oliver Plohmann
 *
 */
public class AskVetoShutdownCallbackHandler extends AbstractVetoableCallbackHandler
{
	protected static final String SYSTEM_NAME = "Distributor"; //$NON-NLS-1$
		
	protected DistributorAnchor anchor = null;
	
	/**
	 * Creates a new <code>AskVetoShutdownCallbackHandler</code> object.
	 */
	public AskVetoShutdownCallbackHandler()
	{
		super(SYSTEM_NAME);
	}

	/**
	  * Creates a new <code>AskVetoShutdownCallbackHandler</code> object.
	  */
	public AskVetoShutdownCallbackHandler(DistributorAnchor anchor)
	{
		super(SYSTEM_NAME);
		this.anchor = anchor;
		
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.events.IVetoableCallbackHandler#askMustVeto(org.almendra.commons.commons.events.Event)
	 */
	public Result askMustVeto(Event event)
	{
		Set<String> workstationNames = anchor.getWorkstationNames();
		if(!workstationNames.isEmpty())
		{			
			StringBuffer text = new StringBuffer();
			text.append("Distributor still connected with nodes! You need to disconnect all executors\n"); //$NON-NLS-1$
			text.append("from their observers before you can shutdown the distributor.\n\n"); //$NON-NLS-1$
			text.append("Ignoring this message will result in socket connections between nodes\n"); //$NON-NLS-1$
			text.append("not to be closed propperly causing RMI exceptions."); //$NON-NLS-1$
			return new Result(text.toString());
		}
		return new Result();
	}
}
