/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.ade.distributor;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.almendra.commons.util.collection.StringList;

/**
 * @author Oliver Plohmann
 *
 */
public class HeaviestLoadedNodeDecisionInformation extends DecisionInformation
{
	protected List<String> nodeNamesToConsider = new StringList();
	protected String heaviestLoadedNodeName = null;
	protected List<String> capabilityPathNames = new StringList();
	protected String heaviestLoadedCapabilityName = null;
	protected Map<String, Integer> cqsByCapabilityName = null;
	protected String nodeNameDroppedToQSC1 = null;
	protected List<String> candidateNodeNamesWithRequiredCapabilities = new StringList();
	
	/**
	 * Creates a new <code>HeaviestLoadedNodeDecisionInformation</code> object.
	 */
	public HeaviestLoadedNodeDecisionInformation()
	{
		super();
	}
	
	/**
	 * Creates a new <code>HeaviestLoadedNodeDecisionInformation</code> object.
	 */
	public HeaviestLoadedNodeDecisionInformation(List<String> nodeNamesToConsider)
	{
		super();
		this.nodeNamesToConsider.addAll(nodeNamesToConsider);
	}

	/**
	 * @param heaviestLoadedNodeName
	 */
	public void setHeaviestLoadedNodeName(String heaviestLoadedNodeName)
	{
		this.heaviestLoadedNodeName = heaviestLoadedNodeName;
	}

	/**
	 * @param capabilities
	 */
	public void setCapabilitiesOfHeaviestLoadedNode(Set<String> capabilityPathNames)
	{
		capabilityPathNames.addAll(capabilityPathNames);
	}

	/**
	 * @param heaviestLoadedCapabilityName
	 */
	public void setHeaviestLoadedCapabilityName(String heaviestLoadedCapabilityName)
	{
		this.heaviestLoadedCapabilityName = heaviestLoadedCapabilityName;
	}

	/**
	 * @param cqsByCapabilityName
	 */
	public void setCQSByCapabilityName(Map<String, Integer> cqsByCapabilityName)
	{
		this.cqsByCapabilityName = new HashMap<String, Integer>(cqsByCapabilityName);
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.ade.distributor.DecisionInformation#printOnPrim(java.lang.StringBuffer, java.lang.String)
	 */
	protected void printOnPrim(StringBuffer text, String tab)
	{
		printNodeNamesAboveQSC1(text, tab);
		printCapabilityPathNames(text, tab);
		printHeaviestLoadedNodeName(text, tab);
		printCQSByCapabilityName(text, tab);
	}
	
	/**
	 * @param tab
	 * @param text
	 */
	protected void printNodeNamesAboveQSC1(StringBuffer text, String tab)
	{
		if(nodeNamesToConsider.isEmpty())
			return;
		text.append(tab);
		text.append("Considered nodes for eviction with QSC >= 2: "); //$NON-NLS-1$
		printOnString(text, nodeNamesToConsider);		
	}
	
	/**
	 * @param tab
	 * @param text
	 */
	protected void printCapabilityPathNames(StringBuffer text, String tab)
	{
		if(capabilityPathNames.isEmpty())
			return;
		text.append(tab);
		text.append("Capability path names: "); //$NON-NLS-1$
		printOnString(text, capabilityPathNames);
	}
	
	protected void printHeaviestLoadedNodeName(StringBuffer text, String tab)
	{
		if(heaviestLoadedNodeName == null)
			return;
		text.append(tab);
		text.append("Heaviest loaded node: " + heaviestLoadedNodeName + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	protected void printHeaviestLoadedCapabilityName(StringBuffer text, String tab)
	{
		if(heaviestLoadedCapabilityName == null)
			return;
		text.append(tab);
		text.append("Heaviest loaded capability: " + heaviestLoadedCapabilityName + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	protected void printCQSByCapabilityName(StringBuffer text, String tab)
	{
		if(cqsByCapabilityName.isEmpty())
			return;
		text.append(tab);
		text.append("CQS on " + heaviestLoadedNodeName + " by capability: "); //$NON-NLS-1$ //$NON-NLS-2$
		printOnStringAssocStringInteger(text, cqsByCapabilityName);
	}

	protected void printNodeNameDroppedToQSC1(StringBuffer text, String tab)
	{
		if(nodeNameDroppedToQSC1 == null)
			return;
		text.append(tab);
		text.append("Node dropped to QSC1: " + nodeNameDroppedToQSC1 + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
	}

	/**
	 * @param tab
	 * @param text
	 */
	protected void printCandidateNodeNamesWithRequiredCapabilities(StringBuffer text, String tab)
	{
		if(candidateNodeNamesWithRequiredCapabilities.isEmpty())
			return;
		text.append(tab);
		text.append("Candidate eviction nodes with QSC >= 2: "); //$NON-NLS-1$
		printOnString(text, candidateNodeNamesWithRequiredCapabilities);
	}
	
	/**
	 * @param allCandidateNodes
	 */
	public void setCandidateNodeNamesWithRequiredCapabilities(Set<String> candidateNodeNamesWithRequiredCapabilities)
	{
		this.candidateNodeNamesWithRequiredCapabilities.addAll(candidateNodeNamesWithRequiredCapabilities);
	}

	/**
	 * @param text
	 * @param tab
	 */
	protected void printHeaviestLoadedNodeNameWithCapabilityName(StringBuffer text, String tab)
	{
		boolean success = true;
		if(heaviestLoadedNodeName == null)
		{
			text.append(tab);
			text.append("No heavily loaded node found!"); //$NON-NLS-1$
			success = false;
		}
		if(heaviestLoadedCapabilityName == null)
		{
			text.append(tab);
			text.append("No heavily loaded capability found!"); //$NON-NLS-1$
			success = false;
		}
		if(!success)
			return;
		text.append(tab);
		text.append("Heaviest loaded capability: " + heaviestLoadedCapabilityName + " on " + heaviestLoadedNodeName + "\n");  //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}

}
