/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.ade.distributor.interpreters;

import java.util.List;

import org.almendra.commons.util.collection.StringList;
import org.almendra.janet.ade.commands.NotifyQSCChangedCommand;
import org.almendra.janet.ade.commands.ProcessQSCChangedCommand;
import org.almendra.janet.ade.distributor.DistributorAnchor;
import org.almendra.janet.cas.scheduling.CommandAccessor;
import org.almendra.janet.cas.scheduling.IInterpreter;



public class NotifyQSCChangedInterpreter extends NotifyValueChangedInterpreter implements IInterpreter
{
	/**
	 * Creates a new <code>NotifyQSCChangedInterpreter</code> object.
	 */
	public NotifyQSCChangedInterpreter()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#execute(org.almendra.commons.janet.cas.scheduling.CommandAccessor)
	 */
	public void execute(CommandAccessor cmdAccessor)
	{
		if(cmdAccessor.getCommand() instanceof NotifyQSCChangedCommand)
		{
			NotifyQSCChangedCommand cmd = (NotifyQSCChangedCommand) cmdAccessor.getCommand();
			DistributorAnchor anchor = getDistributorAnchor(cmdAccessor);
			if(anchor.isDisablePartialEviction())
				return;
			int previousQueueSizeCategory = anchor.getQueueSizeCategory(cmd.getNodeName());
			int currentQueueSizeCategory = cmd.getNewQueueSizeCategory();
			anchor.setNewNodeValues(cmd.getNodeName(), currentQueueSizeCategory, cmd.getWaitingTime(), cmd.getWaitingQueuesByCapability(), cmd.getExecutingQueuesByCapability());
			reflectChange(cmdAccessor, cmd.getNodeName(), previousQueueSizeCategory, currentQueueSizeCategory);
		}
	}

	/**
	 * @param cmdAccessor
	 * @param previousQueueSizeCategory
	 * @param currentQueueSizeCategory
	 */
	protected void reflectChange(CommandAccessor cmdAccessor, String nodeName, int previousQueueSizeCategory, int currentQueueSizeCategory)
	{
		DistributorAnchor anchor = getDistributorAnchor(cmdAccessor);
		if(anchor.isDisableEviction())
			return;
		if(previousQueueSizeCategory > currentQueueSizeCategory && currentQueueSizeCategory < 1)
		{
			if(anchor.isDisableNodeDroppedToQSC1())
				return;
			reflectNodeDroppedToQSC0(nodeName, cmdAccessor, anchor);
			return;
		}
		
		if(currentQueueSizeCategory > 1)
		{
			if(anchor.isDisableNodesWithQSC0Exist())
				return;
			checkImbalances(nodeName, cmdAccessor, anchor);
			return;
		}
	}

	/**
	 * @param nodeName
	 * @param cmdAccessor
	 * @param anchor
	 */
	protected void checkImbalances(String nodeName, CommandAccessor cmdAccessor, DistributorAnchor anchor)
	{
		List<String> nodesWithQSC0 = anchor.getAvailableNodeNamesWithQSC0();
		if(nodesWithQSC0.isEmpty())
			return;
		try
		{
			ProcessQSCChangedCommand command = new ProcessQSCChangedCommand(ProcessQSCChangedCommand.NodesWithQSC0Exist);
			command.setNodeName(nodeName);
			command.setNodeNames(nodesWithQSC0);
			anchor.getBalancerAgent().accept(command);
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			cmdAccessor.getLogger().exception(e);
		}
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#commandNames()
	 */
	public List<String> commandNames()
	{
		return new StringList(NotifyQSCChangedCommand.QualifiedName);
	}
}
