/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.ade.executor;

import java.util.LinkedList;
import java.util.List;

import org.almendra.commons.util.StringUtility;
import org.almendra.commons.util.xml.JDOMUtility;
import org.almendra.commons.util.xml.XMLParseException;
import org.jdom.Element;



public class Descriptor
{
	public static final String TAG_OBSERVER_INTERVAL = "observerInterval"; //$NON-NLS-1$
	public static final String TAG_OBSERVER_INTERVAL_DELAY = "delay"; //$NON-NLS-1$
	public static final String TAG_OBSERVER_INTERVAL_PERIOD = "period"; //$NON-NLS-1$
	
	public static final String TAG_QUEUE_SIZE_CATEGORIES = "queueSizeCategories"; //$NON-NLS-1$
	public static final String TAG_QUEUE_SIZE_LOGEVALUATION = "logEvaluation"; //$NON-NLS-1$
	public static final String TAG_QUEUE_SIZE_CATEGORY = "queueSizeCategory"; //$NON-NLS-1$
	public static final String TAG_QUEUE_SIZE_CATEGORY_NAME = "name"; //$NON-NLS-1$
	public static final String TAG_QUEUE_SIZE_CATEGORY_MAX_WAITING_COMMANDS = "maxWaitingCommands"; //$NON-NLS-1$
	public static final String TAG_QUEUE_SIZE_CATEGORY_MAX_EXECUTING_COMMANDS = "maxExecutingCommands"; //$NON-NLS-1$
	public static final String TAG_QUEUE_SIZE_CONCATENATION_OPERATION = "concatenationOperator"; //$NON-NLS-1$
	
	protected static final int QUEUE_SIZE_CATETGORIES_MIN_SIZE = 3;
	
	protected long delay = -1;
	protected long period = -1;
	protected List<QueueSizeCategory> queueSizeCategories = new LinkedList<QueueSizeCategory>();
	boolean logEvaluation = false;
	
	/**
	 * Creates a new <code>Descriptor</code> object.
	 */
	public Descriptor()
	{
		super();
	}

	/**
	 * @param root
	 * @return
	 */
	public static Descriptor create(Element root) throws XMLParseException
	{
		Descriptor desc = new Descriptor();
		desc.fill(root);
		return desc;
	}

	/**
	 * @param root
	 */
	protected void fill(Element root) throws XMLParseException
	{
		Element element = root.getChild(TAG_OBSERVER_INTERVAL);
		if(element == null)
			throw new XMLParseException("Tag " + TAG_OBSERVER_INTERVAL + " not found parsing ADE_EXECUTOR capability descriptor"); //$NON-NLS-1$ //$NON-NLS-2$
		
		String delayString = element.getAttributeValue(TAG_OBSERVER_INTERVAL_DELAY);
		if(delayString == null)
			throw new XMLParseException("Tag " + TAG_OBSERVER_INTERVAL_DELAY + " not found parsing ADE_EXECUTOR capability descriptor"); //$NON-NLS-1$ //$NON-NLS-2$
		
		Integer number = StringUtility.toInteger(delayString);
		if(number == null)
			throw new XMLParseException("Invalid value " + delayString + " for tag " + TAG_OBSERVER_INTERVAL_DELAY + " found parsing ADE_EXECUTOR capability descriptor"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		delay = number.intValue();
		
		String periodString = element.getAttributeValue(TAG_OBSERVER_INTERVAL_PERIOD);
		if(periodString == null)
			throw new XMLParseException("Tag " + TAG_OBSERVER_INTERVAL_PERIOD + " not found parsing ADE_EXECUTOR capability descriptor"); //$NON-NLS-1$ //$NON-NLS-2$
		
		number = StringUtility.toInteger(delayString);
		if(number == null)
			throw new XMLParseException("Invalid value " + delayString + " for tag " + TAG_OBSERVER_INTERVAL_PERIOD + " found parsing ADE_EXECUTOR capability descriptor"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		period = number.intValue();
		
		fillQueueSizeCategories(root);
	}
	
	/**
	 * @param root
	 */
	protected void fillQueueSizeCategories(Element root) throws XMLParseException
	{
		Element categoriesElement = root.getChild(TAG_QUEUE_SIZE_CATEGORIES);
		if(categoriesElement == null)
			throw new XMLParseException("Tag " + TAG_QUEUE_SIZE_CATEGORIES + " not found parsing ADE_EXECUTOR capability descriptor"); //$NON-NLS-1$ //$NON-NLS-2$
		
		logEvaluation = false;
		String logEvaluationString = categoriesElement.getAttributeValue(TAG_QUEUE_SIZE_LOGEVALUATION);
		if(logEvaluationString != null)
		{
			logEvaluationString = logEvaluationString.toLowerCase();
			if(logEvaluationString.equals("true")) //$NON-NLS-1$
				logEvaluation = true; 
		}
		
		List<Element> categoryElements = JDOMUtility.getChildren(categoriesElement, TAG_QUEUE_SIZE_CATEGORY);
		if(categoryElements == null || categoryElements.isEmpty())
			throw new XMLParseException("Tag " + TAG_QUEUE_SIZE_CATEGORY + " not found parsing ADE_EXECUTOR capability descriptor"); //$NON-NLS-1$ //$NON-NLS-2$
		
		if(categoryElements.size() < QUEUE_SIZE_CATETGORIES_MIN_SIZE)
		{
			StringBuffer buffer = new StringBuffer();
			buffer.append("Error parsing ADE_EXECUTOR capability descriptor"); //$NON-NLS-1$
			buffer.append("Minimal number of entries for tag " + TAG_QUEUE_SIZE_CATEGORY + " is " + QUEUE_SIZE_CATETGORIES_MIN_SIZE); //$NON-NLS-1$ //$NON-NLS-2$
			throw new XMLParseException(buffer.toString());
		}
			
		List<QueueSizeCategory> localQueueSizeCategories = new LinkedList<QueueSizeCategory>();
		
		for(Element categoryElement : categoryElements)
		{
			String nameString = categoryElement.getAttributeValue(TAG_QUEUE_SIZE_CATEGORY_NAME);
			if(nameString == null)
				throw new XMLParseException("Tag " + TAG_QUEUE_SIZE_CATEGORY_NAME + " not found in " + TAG_QUEUE_SIZE_CATEGORIES + " parsing ADE_EXECUTOR capability descriptor"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			
			Integer nameInt = StringUtility.toInteger(nameString);
			if(nameInt == null)
				throw new XMLParseException("Invalid value " + nameInt + " for tag " + TAG_QUEUE_SIZE_CATEGORY_NAME + " found parsing ADE_EXECUTOR capability descriptor"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			int name = nameInt.intValue();				
			
			String maxExecutingCommandsString = categoryElement.getAttributeValue(TAG_QUEUE_SIZE_CATEGORY_MAX_EXECUTING_COMMANDS);
			if(maxExecutingCommandsString == null)
				throw new XMLParseException("Tag " + TAG_QUEUE_SIZE_CATEGORY_MAX_EXECUTING_COMMANDS + " not found in " + TAG_QUEUE_SIZE_CATEGORIES + " parsing ADE_EXECUTOR capability descriptor"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			
			int maxExecutingCommands = -1;
			if(!StringUtility.isAsterisk(maxExecutingCommandsString))
			{
				Integer maxExecutingCommandsInt = StringUtility.toInteger(maxExecutingCommandsString);
				if(maxExecutingCommandsInt == null)
					throw new XMLParseException("Invalid value " + maxExecutingCommandsString + " for tag " + TAG_QUEUE_SIZE_CATEGORY_MAX_EXECUTING_COMMANDS + " found parsing ADE_EXECUTOR capability descriptor"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				maxExecutingCommands = maxExecutingCommandsInt.intValue();
			}
			
			String maxWaitingCommandsString = categoryElement.getAttributeValue(TAG_QUEUE_SIZE_CATEGORY_MAX_WAITING_COMMANDS);
			if(maxWaitingCommandsString == null)
				throw new XMLParseException("Tag " + TAG_QUEUE_SIZE_CATEGORY_MAX_WAITING_COMMANDS + " not found in " + TAG_QUEUE_SIZE_CATEGORIES + " parsing ADE_EXECUTOR capability descriptor"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			
			int maxWaitingCommands = -1;
			if(!StringUtility.isAsterisk(maxWaitingCommandsString))
			{
				Integer maxWaitingCommandsInt = StringUtility.toInteger(maxWaitingCommandsString);
				if(maxWaitingCommandsInt == null)
					throw new XMLParseException("Invalid value " + maxWaitingCommandsString + " for tag " + TAG_QUEUE_SIZE_CATEGORY_MAX_WAITING_COMMANDS + " found parsing ADE_EXECUTOR capability descriptor"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				maxWaitingCommands = maxWaitingCommandsInt.intValue();
			}
			
			boolean concatenationIsOr = false;
			String concatenationOperatorString = categoryElement.getAttributeValue(TAG_QUEUE_SIZE_CONCATENATION_OPERATION);
			if(concatenationOperatorString == null)
				throw new XMLParseException("Tag " + TAG_QUEUE_SIZE_CONCATENATION_OPERATION + " not found in " + TAG_QUEUE_SIZE_CATEGORIES + " parsing ADE_EXECUTOR capability descriptor"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			
			boolean opFound = false;
			concatenationOperatorString = concatenationOperatorString.toLowerCase();
			if(concatenationOperatorString.equals("or")) //$NON-NLS-1$
			{
				concatenationIsOr = true;
				opFound = true;
			}
			if(concatenationOperatorString.equals("and")) //$NON-NLS-1$
			{
				concatenationIsOr = false;
				opFound = true;
			}
			if(!opFound)
				throw new XMLParseException("Value of tag  " + TAG_QUEUE_SIZE_CONCATENATION_OPERATION + " in ADE_EXECUTOR capability descriptor invalid: must be \"or\" or \"and\"!"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
						
			QueueSizeCategory category = new QueueSizeCategory(name, maxWaitingCommands, maxExecutingCommands, concatenationIsOr); 
						
			localQueueSizeCategories.add(category);
		}
		
		queueSizeCategories.addAll(localQueueSizeCategories);
	}

	/**
	 * @return Returns the delay.
	 */
	public long getDelay()
	{
		if(delay == -1)
			return 3000;
		return delay;
	}
	
	/**
	 * @return Returns the period.
	 */
	public long getPeriod()
	{
		if(period == -1)
			return 3000;
		return period;
	}
	/**
	 * @return Returns the queueSizeCategories.
	 */
	public List<QueueSizeCategory> getQueueSizeCategories()
	{
		return queueSizeCategories;
	}
	/**
	 * @return Returns the logEvaluation.
	 */
	public boolean isLogEvaluation()
	{
		return logEvaluation;
	}
}
