/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.ade.executor.executors;

import java.util.List;

import org.almendra.commons.threads.IExecutable;
import org.almendra.janet.ade.commands.ExecuteWorkloadCommand;
import org.almendra.janet.ade.executor.ExecutorAnchor;
import org.almendra.janet.cas.node.AgentPath;
import org.almendra.janet.cas.node.SystemApplication;
import org.almendra.janet.cas.scheduling.ICommandEnvelope;



public class EvictWaitCommandsExecutor extends EvictCommandsExecutor implements IExecutable
{
	/**
	 * Creates a new <code>EvictWaitCommandsExecutor</code> object.
	 */
	protected EvictWaitCommandsExecutor()
	{
		super();
	}
	
	/**
	 * Creates a new <code>EvictWaitCommandsExecutor</code> object.
	 */
	public EvictWaitCommandsExecutor(String token)
	{
		super(token);
	}
	
	/**
	 * Creates a new <code>EvictWaitCommandsExecutor</code> object.
	 */
	public EvictWaitCommandsExecutor(SystemApplication application, AgentPath evictionAgent, String capabilityPath, String token)
	{
		super(application, evictionAgent, capabilityPath, token);
	}
	
	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.threads.IExecutable#execute(java.lang.Object)
	 */
	public void execute(Object object)
	{	
		anchor = (ExecutorAnchor) object;	
		evict1CommandFromCapability();
	}
	
	/**
	 * @param object
	 */
	protected boolean evict1CommandFromCapability()
	{				
		evictionAgent = getEvictionAgent();
		if(evictionAgent == null)
			return false;
		
		List<ICommandEnvelope> commands = anchor.getWaitingCommandsByCapability().get(capabilityPath);		
		if(commands == null || commands.isEmpty())
		{
			anchor.logNoWaitingCommandToEvict();
			return false;
		}			
		
		ICommandEnvelope env = getNextCommand(commands);
		if(env == null) 
		{
			anchor.logNoWaitingCommandToEvict();
			return false;
		}
		
		sendToEvictionAgent(anchor, env);
		
		incrementEvictionCommandsCount();
		anchor.evictedWaitingCommand(env, capabilityPath);
		return true;
	}

	/**
	 * @param evictionAgent
	 * @param env
	 */
	protected boolean sendToEvictionAgent(ExecutorAnchor anchor, ICommandEnvelope env)
	{
		try
		{
			setEvictionAgent();
			env.setNewDestination(evictionAgentPath, anchor.getNodeName());
			prepareEnvelopeBeforeSending(env);			
			evictionAgent.accept(env);			
			anchor.logEvictedWaitingCommand(env.getCommand().toLogString());
			return true;
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			getLogger().exception(e);
		}
		return false;
	}

	/**
	 * 
	 */
	protected void setEvictionAgent()
	{
		// redefine in subclass as appropriate
	}

	/**
	 * @param env
	 */
	protected void prepareEnvelopeBeforeSending(ICommandEnvelope env)
	{
		ExecuteWorkloadCommand workloadCommand = (ExecuteWorkloadCommand) env.getCommand();
		workloadCommand.prepareBeforeSending();
		env.setPredefinedInterpreter(null);
	}

	/**
	 * @param env
	 */
	protected void setSuspended(ICommandEnvelope env)
	{
		env.removeFromQueue();
	}
}
