/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.ade.executor.handlers;

import org.almendra.commons.events.Event;
import org.almendra.commons.events.IEventHandler;
import org.almendra.janet.ade.commands.ExecuteWorkloadCommand;
import org.almendra.janet.ade.commands.WorkloadDistributionCommand;
import org.almendra.janet.ade.executor.AnchorHolder;
import org.almendra.janet.ade.executor.ExecutorAnchor;
import org.almendra.janet.cas.agents.IAgentProxy;
import org.almendra.janet.cas.handlers.CommandEnvelopeParams;
import org.almendra.janet.cas.node.AgentPath;
import org.almendra.janet.cas.node.LocalCapabilityPath;
import org.almendra.janet.cas.scheduling.ICommandEnvelope;



public class CommandArrivedHandler extends AnchorHolder implements IEventHandler
{	
	/**
	 * Creates a new <code>CommandArrivedHandler</code> object.
	 */
	protected CommandArrivedHandler()
	{
		super();
	}
	
	/**
	 * Creates a new <code>CommandArrivedHandler</code> object.
	 */
	public CommandArrivedHandler(ExecutorAnchor anchor)
	{
		super(anchor);
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.events.IEventHandler#occured(org.almendra.commons.commons.events.Event)
	 */
	public void occured(Event event)
	{
		if(event.getParams() instanceof CommandEnvelopeParams)
		{
			CommandEnvelopeParams params = (CommandEnvelopeParams) event.getParams();
			ICommandEnvelope env = params.getCommandEnvelope();
			
			if(!env.isWorkloadCommand())
				return;
			
			ExecuteWorkloadCommand executeCommand = (ExecuteWorkloadCommand) env.getCommand();
			
			if(!executeCommand.isMustAccept() && anchor.isAboveThreshold())
			{
				if(sendToLoadSharingAgent(env))
					return;
			}
						
			executeCommand.setMustAccept(false);
			executeCommand.setCurrentEnvelope(env);
			executeCommand.setAnchor(anchor);
			String capabilityPath = executeCommand.getCapabilityPath();
			if(env.isForeign())
				setLeastLoadedDestinationAgent(env, capabilityPath);				
			env.setPredefinedInterpreter(executeCommand.getInterpreter());
			executeCommand.workloadCommandArrived(anchor, env, capabilityPath); 						
		}
	}

	/**
	 * @param env
	 * @return
	 */
	protected boolean sendToLoadSharingAgent(ICommandEnvelope env)
	{
		IAgentProxy agent = anchor.getNextLoadSharingAgent();
		if(agent == null)
			return false;
		try
		{
			AgentPath path = anchor.getLoadAdministratorAgentPath(agent);
			ExecuteWorkloadCommand command = (ExecuteWorkloadCommand) env.getCommand();
			env.setNewDestination(path, path.getNodeName());
			env.setCommand(new WorkloadDistributionCommand(env.getOriginAgentPath(), command));
			agent.accept(env);
			env.setIgnore(true);
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			anchor.getLogger().exception(e);
			return false;
		}
		return true;
	}

	/**
	 * @param env
	 * @param path
	 */
	protected void setLeastLoadedDestinationAgent(ICommandEnvelope env, String capabilityPath)
	{
		String agentName = anchor.getLeastLoadedAgentName(new LocalCapabilityPath(capabilityPath));
		if(agentName != null)
			env.getDestinationAgent().setAgentName(agentName);
	}
}
