/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.ade.observer;

import org.almendra.commons.util.StringUtility;
import org.almendra.commons.util.xml.XMLParseException;
import org.jdom.Element;



public class Descriptor
{
	public static final String TAG_OBSERVER_INTERVAL = "interval"; //$NON-NLS-1$
	public static final String TAG_OBSERVER_PERIOD = "periodInMillis"; //$NON-NLS-1$
	public static final String TAG_LOAD_CHANGE = "loadChange"; //$NON-NLS-1$
	public static final String TAG_LOAD_CHANGE_SIGNIFICANT_WHEN_EXCEEDING = "significantWhenChangeExceedingPercentage"; //$NON-NLS-1$
	public static final String TAG_LOAD_CHANGE_EVICTION_THRESHOLD = "evictionThresholdPercentage"; //$NON-NLS-1$	
	public static final String TAG_LOAD_CHANGE_SIGNIFICANT_AFTER = "afterMillis"; //$NON-NLS-1$
	
	protected long period = -1;
	protected  int significantWhenChangeExceeding = -1;
	protected  int after = -1;
	protected  int evictionThreshold = -1;	
	
	/**
	 * Creates a new <code>Descriptor</code> object.
	 */
	public Descriptor()
	{
		super();
	}

	/**
	 * @param root
	 * @return
	 */
	public static Descriptor create(Element root) throws XMLParseException
	{
		Descriptor desc = new Descriptor();
		desc.fill(root);
		return desc;
	}

	/**
	 * @param root
	 */
	protected void fill(Element root) throws XMLParseException
	{
		Element element = root.getChild(TAG_OBSERVER_INTERVAL);
		if(element == null)
			throw new XMLParseException("Tag " + TAG_OBSERVER_INTERVAL + " not found parsing ADE_OBSERVER capability descriptor"); //$NON-NLS-1$ //$NON-NLS-2$
		
		String periodString = element.getAttributeValue(TAG_OBSERVER_PERIOD);
		if(periodString == null)
			throw new XMLParseException("Tag " + TAG_OBSERVER_PERIOD + " not found parsing ADE_OBSERVER capability descriptor"); //$NON-NLS-1$ //$NON-NLS-2$
		
		Integer number = StringUtility.toInteger(periodString);
		if(number == null)
			throw new XMLParseException("Invalid value " + periodString + " for tag " + TAG_OBSERVER_PERIOD + " found parsing ADE_OBSERVER capability descriptor"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		period = number.intValue();
		
		element = root.getChild(TAG_LOAD_CHANGE);
		if(element == null)
			throw new XMLParseException("Tag " + TAG_LOAD_CHANGE + " not found parsing ADE_OBSERVER capability descriptor"); //$NON-NLS-1$ //$NON-NLS-2$
		
		String significantWhenExceedingPercentageString = element.getAttributeValue(TAG_LOAD_CHANGE_SIGNIFICANT_WHEN_EXCEEDING);
		if(significantWhenExceedingPercentageString == null)
			throw new XMLParseException("Tag " + TAG_LOAD_CHANGE_SIGNIFICANT_WHEN_EXCEEDING + " not found parsing ADE_OBSERVER capability descriptor"); //$NON-NLS-1$ //$NON-NLS-2$
		
		number = StringUtility.toInteger(significantWhenExceedingPercentageString);
		if(number == null)
			throw new XMLParseException("Invalid value " + significantWhenExceedingPercentageString + " for tag " + TAG_LOAD_CHANGE_SIGNIFICANT_WHEN_EXCEEDING + " found parsing ADE_OBSERVER capability descriptor"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		significantWhenChangeExceeding = number.intValue();
				
		String afterMillisString = element.getAttributeValue(TAG_LOAD_CHANGE_SIGNIFICANT_AFTER);
		if(afterMillisString == null)
			throw new XMLParseException("Tag " + TAG_LOAD_CHANGE_SIGNIFICANT_AFTER + " not found parsing ADE_OBSERVER capability descriptor"); //$NON-NLS-1$ //$NON-NLS-2$
		
		number = StringUtility.toInteger(afterMillisString);
		if(number == null)
			throw new XMLParseException("Invalid value " + afterMillisString + " for tag " + TAG_LOAD_CHANGE_SIGNIFICANT_AFTER + " found parsing ADE_OBSERVER capability descriptor"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		after = number.intValue();	
		
		String evictionThresholdString = element.getAttributeValue(TAG_LOAD_CHANGE_EVICTION_THRESHOLD);
		if(number == null)
			throw new XMLParseException("Tag " + TAG_LOAD_CHANGE_EVICTION_THRESHOLD + " not found parsing ADE_OBSERVER capability descriptor"); //$NON-NLS-1$ //$NON-NLS-2$
		
		number = StringUtility.toInteger(evictionThresholdString);
		if(number == null)
			throw new XMLParseException("Invalid value " + evictionThresholdString + " for tag " + TAG_LOAD_CHANGE_EVICTION_THRESHOLD + " found parsing ADE_OBSERVER capability descriptor"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		evictionThreshold = number.intValue();
	}
	
	/**
	 * @return Returns the period.
	 */
	public long getPeriod()
	{
		if(period == -1)
			return 2000;
		return period;
	}
	
	/**
	 * @return Returns the after.
	 */
	public int getAfter()
	{
		return after;
	}
	
	/**
	 * @return Returns the significantWhenChangeExceeding.
	 */
	public int getSignificantWhenChangeExceeding()
	{
		return significantWhenChangeExceeding;
	}
	
	/**
	 * @return Returns the evictionThreshold.
	 */
	public int getEvictionThreshold()
	{
		return evictionThreshold;
	}
}
