/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.logger.interpreters;

import java.util.List;

import org.almendra.commons.util.collection.StringList;
import org.almendra.janet.cas.logger.LoggerServerAnchor;
import org.almendra.janet.cas.logger.commands.DisplayLogEntryCommand;
import org.almendra.janet.cas.scheduling.CommandAccessor;
import org.almendra.janet.cas.scheduling.IInterpreter;


/**
 * @author Oliver
 *
 */
public class DisplayLogEntryInterpreter extends LoggerInterpreter implements IInterpreter
{

	/**
	 * Creates a new <code>DisplayLogEntryInterpreter</code> object.
	 */
	public DisplayLogEntryInterpreter()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.schedulers.IInterpreter#execute(org.almendra.commons.janet.cas.logger.schedulers.CommandAccessor)
	 */
	public void execute(CommandAccessor cmdAccessor)
	{
		if(cmdAccessor.getCommand() instanceof DisplayLogEntryCommand)
		{
			DisplayLogEntryCommand cmd = (DisplayLogEntryCommand) cmdAccessor.getCommand();
			LoggerServerAnchor anchor = getLoggerServerAnchor(cmdAccessor);
			anchor.getController().log(cmd.getEntries());
		}
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#commandNames()
	 */
	public List<String> commandNames()
	{
		return new StringList(DisplayLogEntryCommand.QualifiedName);
	}

}
