/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.logger.interpreters;

import java.util.List;

import org.almendra.commons.events.IEventHandler;
import org.almendra.commons.util.collection.StringList;
import org.almendra.janet.cas.agents.IAgentProxy;
import org.almendra.janet.cas.events.EventHandlerEnvelope;
import org.almendra.janet.cas.events.HandlerList;
import org.almendra.janet.cas.handlers.ForeignApplicationRegisteredParams;
import org.almendra.janet.cas.logger.LoggerAnchor;
import org.almendra.janet.cas.logger.LoggerServerAnchor;
import org.almendra.janet.cas.logger.commands.InstallLoggerCommand;
import org.almendra.janet.cas.logger.commands.StartViewCommand;
import org.almendra.janet.cas.logger.gui.LoggerView;
import org.almendra.janet.cas.logger.gui.LoggerViewController;
import org.almendra.janet.cas.logger.handlers.ForeignApplicationRegisteredHandler;
import org.almendra.janet.cas.scheduling.CommandAccessor;
import org.almendra.janet.cas.scheduling.IInterpreter;


/**
 * @author Oliver
 *
 */
public class StartViewInterpreter extends LoggerInterpreter implements IInterpreter
{

	/**
	 * Creates a new <code>StartViewInterpreter</code> object.
	 */
	public StartViewInterpreter()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.schedulers.IInterpreter#execute(org.almendra.commons.janet.cas.logger.schedulers.CommandAccessor)
	 */
	public void execute(CommandAccessor cmdAccessor)
	{
		if(cmdAccessor.getCommand() instanceof StartViewCommand)
		{				
			LoggerViewController controller = LoggerView.open(cmdAccessor.getNodeAccessor(), cmdAccessor.getApplication());
			installServerAnchor(cmdAccessor, controller);
			installLoggers(cmdAccessor);
			registerForEvents(cmdAccessor, controller);
		}
	}

	protected void registerForEvents(CommandAccessor cmdAccessor, LoggerViewController controller)
	{
		registerForeignApplicationRegistered(cmdAccessor, controller);
	}

	protected void installServerAnchor(CommandAccessor cmdAccessor, LoggerViewController controller)
	{
		cmdAccessor.setNodeLabel(LoggerAnchor.NODE_LABEL);
		cmdAccessor.getObjectSpace().put(LoggerServerAnchor.LABEL, new LoggerServerAnchor(controller));
	}

	protected void registerForeignApplicationRegistered(CommandAccessor cmdAccessor, LoggerViewController viewController)
	{
		IEventHandler handler = new ForeignApplicationRegisteredHandler(viewController, cmdAccessor.getCurrentAgent());
		EventHandlerEnvelope env = new EventHandlerEnvelope(handler, cmdAccessor.getAgentPath()); 
		HandlerList handlers = new HandlerList(env);
		cmdAccessor.getNodeEventRegistry().register(
			ForeignApplicationRegisteredParams.EVENT_NAME, 
			handlers,
			cmdAccessor.getApplication());
	}

	/**
	 * @param cmdAccessor
	 */
	protected void installLoggers(CommandAccessor cmdAccessor)
	{
		try
		{
			InstallLoggerCommand cmd = new InstallLoggerCommand(cmdAccessor.getCurrentAgent().getAgentPath());
			IAgentProxy agents = cmdAccessor.getCurrentAgent().getAllForeignAgents(LoggerAnchor.CAPABILITY_PRODUCER_NAME);
			agents.accept(cmd);
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			cmdAccessor.getLogger().exception(e);
		}
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#commandNames()
	 */
	public List<String> commandNames()
	{
		return new StringList(StartViewCommand.QualifiedName);
	}

}
