/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.cas.monitor.commands;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;

import org.almendra.commons.util.AssertUtility;
import org.almendra.janet.cas.scheduling.ICommand;



public class DisplayAttributeCommand extends AbstractConsumerCommand implements ICommand, Serializable
{

	private static final long serialVersionUID = 8364127663213900545L;

	public static final String QualifiedName = "org.almendra.commons.janet.cas.monitor.commands.DisplayAttributeCommand"; //$NON-NLS-1$
	
	public static final int OperationIncrement = 1;
	public static final int OperationDecrement = 2;
	
	protected List<Entry> entries = new LinkedList<Entry>();
	/**
	 * Creates a new <code>DisplayAttributeCommand</code> object.
	 */
	protected DisplayAttributeCommand()
	{
		super();
	}
	
	public DisplayAttributeCommand(String nodeName)
	{
		super(nodeName);
		AssertUtility.assertNotNull(nodeName);
	}
	
	public DisplayAttributeCommand(String nodeName, List<Entry> entries)
	{
		super(nodeName);
		AssertUtility.assertNotNull(nodeName);
		this.entries = entries;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.ICommand#getQualifiedName()
	 */
	public String getQualifiedName()
	{
		return QualifiedName;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.ICommand#getPriority()
	 */
	public int getPriority()
	{
		return ICommand.ApplicationMaxPriority;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.ICommand#toLogString()
	 */
	public String toLogString()
	{
		return "DisplayAttributeCommand"; //$NON-NLS-1$
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	public Object clone() throws CloneNotSupportedException
	{
		DisplayAttributeCommand cmd = (DisplayAttributeCommand) super.clone();
		return cmd;
	}

	
	public class Entry implements Serializable
	{
		private static final long serialVersionUID = 4786669652184380004L;
		public String attribute = null;
		public String value = null;
		public int operation = -1;
		
		public boolean isIncrement()
		{
			return value == null && operation == OperationIncrement;
		}
		
		public boolean isDecrement()
		{
			return value == null && operation == OperationDecrement;
		}
	}


	/**
	 * @param entry
	 */
	public void add(Entry entry)
	{
		entries.add(entry);
	}
	
	public void add(List<Entry> entries)
	{
		this.entries.addAll(entries);
	}
	
	/**
	 * @return Returns the entries.
	 */
	public List<Entry> getEntries()
	{
		return entries;
	}
}
