/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.simulator.node.interpreters.simulated;

import java.rmi.AlreadyBoundException;

import org.almendra.commons.util.inet.RegistryLocation;
import org.almendra.janet.cas.commands.NodeStartedCommand;
import org.almendra.janet.cas.node.CentralAnchor;
import org.almendra.janet.cas.node.IAgentDispatcher;
import org.almendra.janet.cas.node.Node;
import org.almendra.janet.cas.simulator.Registry;


/**
 * @author Oliver
 *  
 */
public class NodeStartedInterpreter extends org.almendra.janet.cas.node.interpreters.NodeStartedInterpreter
{

	public NodeStartedInterpreter()
	{
		super();
	}

	protected boolean primBind(String nodeName, NodeStartedCommand command)
	{
		boolean success = false;
		try
		{
			Registry.get().bind(Node.RMI_LABEL_PREFIX + nodeName, command.getNode().getAgentDispatcher());
			success = true;
		}
		catch (AlreadyBoundException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
		return success;
	}
	
	protected String[] registryList(RegistryLocation registryLocation)
	{
		return Registry.get().list();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.commands.NodeStartedInterpreter#getCentralDispatcher(org.almendra.commons.commons.util.inet.RegistryLocation)
	 */
	protected IAgentDispatcher getCentralDispatcher(RegistryLocation registryLocation)
	{
		return (IAgentDispatcher) Registry.get().lookup(CentralAnchor.RMI_NAME);
	}

}
