/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
 
package org.almendra.janet.cas.agents;

import org.almendra.janet.cas.node.Logger;
import org.almendra.janet.cas.node.Node;
import org.almendra.janet.cas.node.SystemApplication;
import org.almendra.janet.cas.schedulers.CommandScheduler;


/**
 * 
 * @author Oliver Plohmann
 */
public class SystemAgent extends AbstractAgent
{
	public static final String NAME = "SYSTEM_AGENT";  //$NON-NLS-1$
	public static final String CAPABILITY_NAME = SystemApplication.CORE_CAPABILITY_NAME;
	
	protected CommandScheduler scheduler = null;

	protected SystemAgent()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.cas.node.AbstractAgent#getScheduler()
	 */
	protected CommandScheduler getScheduler()
	{
		return scheduler;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.cas.node.AbstractAgent#setScheduler(org.almendra.commons.cas.node.scheduling.CommandScheduler)
	 */
	public void setScheduler(CommandScheduler scheduler)
	{
		this.scheduler = scheduler;		
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.cas.node.AbstractAgent#schedulerStopped()
	 */
	protected void schedulerStopped()
	{
		// TODO Auto-generated method stub
		
	}

	/**
	 * @param node
	 * @param logger
	 * @param name
	 */
	public SystemAgent(Node node, SystemApplication systemApplication, Logger logger)
	{
		super(node, logger, NAME);
		scheduler = node.getSystemScheduler();
		systemApplication.setAndInitLogger(logger);
		systemApplication.setNode(node);
		node.getApplicationRegistry().registerSystemApplication(systemApplication);
		capability = systemApplication.getCoreCapability();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.node.AbstractAgent#isSystemAgent()
	 */
	public boolean isSystemAgent()
	{
		return true;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.node.AbstractAgent#prepareFinalize()
	 */
	protected void prepareFinalize()
	{
		super.prepareFinalize();
		scheduler = null;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.agents.AbstractAgent#isApplicationAgent()
	 */
	public boolean isApplicationAgent()
	{
		return false;
	}
	
	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.agents.AbstractAgent#isIdle()
	 */
	public boolean isIdle()
	{
		return scheduler.isIdle();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.agents.AbstractAgent#getQueueSize()
	 */
	public long getQueueSize()
	{
		return scheduler.getQueueSize();
	}

}
