/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.commands;

import java.io.Serializable;
import java.util.List;

import org.almendra.janet.cas.node.AgentPath;
import org.almendra.janet.cas.node.NodeLocation;
import org.almendra.janet.cas.scheduling.ICommand;


public class RegisterApplicationCommand implements ICommand, Serializable
{
	private static final long serialVersionUID = 5480129518597088933L;

	public static final String QualifiedName = "org.almendra.commons.cas.node.commands.RegisterApplicationCommand"; //$NON-NLS-1$
	
	protected List<AgentPath> agentPaths = null;
	protected String originNodeName = null;
	protected NodeLocation nodeLocation = null;
	
	/**
	 * Creates a new <code>RegisterApplicationCommand</code> object.
	 * 
	 */
	protected RegisterApplicationCommand()
	{
		super();
	}
	
	/**
	  * Creates a new <code>RegisterApplicationCommand</code> object.
	  */
	public RegisterApplicationCommand(String originNodeName, List<AgentPath> agentPaths, NodeLocation nodeLocation)
	{
		super();
		this.originNodeName = originNodeName;
		this.agentPaths = agentPaths;
		this.nodeLocation = nodeLocation;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.ICommand#getQualifiedName()
	 */
	public String getQualifiedName()
	{
		return QualifiedName;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.ICommand#getPriority()
	 */
	public int getPriority()
	{
		return ICommand.SystemPriority;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.ICommand#toLogString()
	 */
	public String toLogString()
	{
		return "RegisterApplicationCommand"; //$NON-NLS-1$
	}

	/**
	 * @return Returns the agentPaths.
	 */
	public List<AgentPath> getAgentPaths()
	{
		return agentPaths;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	public Object clone() throws CloneNotSupportedException
	{
		RegisterApplicationCommand cmd = (RegisterApplicationCommand) super.clone();
		return cmd;
	}
	
	/**
	 * @return Returns the sendingNodeName.
	 */
	public String getOriginNodeName()
	{
		return originNodeName;
	}
	/**
	 * @return Returns the nodeLocation.
	 */
	public NodeLocation getNodeLocation()
	{
		return nodeLocation;
	}
}
