/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.cas.events;

import java.util.ArrayList;
import java.util.List;

import org.almendra.commons.events.EventNotifier;
import org.almendra.commons.util.collection.IIdentifiableListEvaluator;
import org.almendra.commons.util.collection.IListEvaluator;


public class ClusterEventNotifier<T extends ClusterEventHandlerEnvelope> extends EventNotifier<T>
{

    /**
	 * List of event handlers to be invoked when event occurs.
	 */
	protected IIdentifiableListEvaluator<T> identifyableEventHandlers = null;
    
	/**
	 * Creates a new <code>ClusterEventNotifier</code> object.
	 */
	public ClusterEventNotifier(IIdentifiableListEvaluator<T> eventHandlers)
	{
	    identifyableEventHandlers = eventHandlers;
	}

	/**
	 * @param handlerListId
	 * @return
	 */
	public boolean hasHandlerList(String handlerListId)
	{
		return identifyableEventHandlers.getId().equals(handlerListId);
	}

	/**
	 * @param originNodeName
	 * @param event
	 * @return
	 */
	public boolean isRegistered(String nodeName)
	{
		for(T env : getEventHandlers().get())
		{
			if(env.getAgentPath().getNodeName().equals(nodeName))
				return true;
		}
		return false;
	}

	/**
	 * @param nodeName
	 */
	protected boolean removeHandlers(String nodeName)
	{
		boolean found = false;
		List<T> copiedEnvelopes = new ArrayList<T>(getEventHandlers().get());
		for(ClusterEventHandlerEnvelope env : copiedEnvelopes)
		{
			System.out.println("env.getAgentPath=" + env.getAgentPath()); //$NON-NLS-1$
			System.out.println("env.getHandler()=" + env.getHandler());						 //$NON-NLS-1$
			System.out.println("env.getAgentPath().getNodeName()=" + env.getAgentPath().getNodeName()); //$NON-NLS-1$
			if(env.getAgentPath().getNodeName().equals(nodeName))
				found |= getEventHandlers().get().remove(env);
		}
		return found;
	}

	/**
	 * @return Returns the eventHandlers.
	 */
	public IListEvaluator<T> getEventHandlers()
	{
		return identifyableEventHandlers;
	}
}
