/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.events;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.NoSuchElementException;

import org.almendra.commons.events.Event;
import org.almendra.commons.util.collection.IIdentifiableListEvaluator;
import org.almendra.commons.util.exception.PropagatedException;
import org.almendra.commons.util.inet.RegistryLocation;


public class ClusterEventRegistryRemote extends UnicastRemoteObject implements IClusterEventRegistry
{
	private static final long serialVersionUID = -4653626570993321559L;
	protected ClusterEventRegistry<ClusterEventNotifier<ClusterEventHandlerEnvelope>> eventRegistry = new ClusterEventRegistry<ClusterEventNotifier<ClusterEventHandlerEnvelope>>();
	
	/**
	 * Creates a new <code>ClusterEventRegistryRemote</code> object.
	 */
	protected ClusterEventRegistryRemote() throws RemoteException
	{
		super();
	}
	
	/**
	 * Creates a new <code>ClusterEventRegistryRemote</code> object.
	 */
	public ClusterEventRegistryRemote(ClusterEventRegistry<ClusterEventNotifier<ClusterEventHandlerEnvelope>> eventRegistry) throws RemoteException
	{
		super();
		this.eventRegistry = eventRegistry;
	}		

	/**
	 * @param nodeName
	 */
	public boolean deregister(String nodeName)
	{
		return eventRegistry.deregister(nodeName);
	}

	/**
	 * @param eventName
	 * @param handlerListId
	 * @throws NoSuchElementException
	 * @throws IllegalArgumentException
	 */
	public boolean deregister(String eventName, String handlerListId) throws NoSuchElementException, IllegalArgumentException
	{
		return eventRegistry.deregister(eventName, handlerListId);
	}

	/**
	 * @param event
	 * @throws PropagatedException
	 */
	public void occured(String originNodeName, Event event) throws PropagatedException
	{
		eventRegistry.occured(originNodeName, event);
	}

	/**
	 * @param nodeName
	 */
	public void register(String nodeName, RegistryLocation registryLocation)
	{
		eventRegistry.register(nodeName, registryLocation);
	}

	/**
	 * @param eventName
	 * @param handlerList
	 * @throws IllegalArgumentException
	 * @throws RemoteException
	 */
	public void register(String eventName, IIdentifiableListEvaluator<ClusterEventHandlerEnvelope> list) throws IllegalArgumentException, RemoteException
	{
		eventRegistry.register(eventName, list);
	}

	/**
	 * @param eventName
	 * @return
	 */
	public boolean isRegistered(String eventName)
	{
		return eventRegistry.isRegistered(eventName);
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.events.IClusterEventRegistry#getHandlerCount(java.lang.String)
	 */
	public int getHandlerCount(String eventName)
	{
		return eventRegistry.getHandlerCount(eventName);
	}

}
