/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.handlers;

import java.util.ArrayList;
import java.util.List;

import org.almendra.commons.events.AbstractEventParams;
import org.almendra.commons.events.IEventParams;
import org.almendra.commons.threads.FutureResult;
import org.almendra.commons.util.inet.RegistryLocation;
import org.almendra.janet.cas.node.ClusterEventRegistryDescriptor;
import org.almendra.janet.cas.node.ClusterSpaceDescriptor;


/**
 * @author Oliver
 *
 */

public class NodeStartupEventParams extends AbstractEventParams implements IEventParams
{
	private static final long serialVersionUID = 5827202695708195739L;

	public static final String EVENT_NAME = "EVENT_NODE_STARTED"; //$NON-NLS-1$
	
	protected FutureResult future = null;
	protected RegistryLocation centralRegistryLocation = null;
	protected RegistryLocation localRegistryLocation = null;
	protected List<ClusterSpaceDescriptor> clusterSpaceDescriptors = new ArrayList<ClusterSpaceDescriptor>();
	protected List<ClusterEventRegistryDescriptor> clusterEventRegistryDescriptors = new ArrayList<ClusterEventRegistryDescriptor>();
	
	protected boolean startGUI = false;
	
	/**
	 * @return Returns the future.
	 */
	public FutureResult getFuture()
	{
		return future;
	}

	private NodeStartupEventParams()
	{
		super();
	}
	
	public NodeStartupEventParams(
			FutureResult future, 
			RegistryLocation centralRegistryLocation, 
			RegistryLocation localRegistryLocation,
			List<ClusterSpaceDescriptor> clusterSpaceDescriptors,
			List<ClusterEventRegistryDescriptor> clusterEventRegistryDescriptors,
			boolean startGUI)
	{
		super();
		this.future = future;
		this.centralRegistryLocation = centralRegistryLocation;
		this.localRegistryLocation = localRegistryLocation;
		this.clusterSpaceDescriptors = clusterSpaceDescriptors;
		this.clusterEventRegistryDescriptors = clusterEventRegistryDescriptors;
		this.startGUI = startGUI;
	}

	/**
	 * @return Returns the centralRegistryLocation.
	 */
	public RegistryLocation getCentralRegistryLocation()
	{
		return centralRegistryLocation;
	}

	/**
	 * @return Returns the localRegistryLocation.
	 */
	public RegistryLocation getLocalRegistryLocation()
	{
		return localRegistryLocation;
	}

	/**
	 * @return Returns the startGUI.
	 */
	public boolean isStartGUI()
	{
		return startGUI;
	}

	/**
	 * @return
	 */
	public List<ClusterSpaceDescriptor> getClusterSpaceDescriptors()
	{
		return clusterSpaceDescriptors;
	}

	/**
	 * @return Returns the clusterEventRegistryDescriptors.
	 */
	public List<ClusterEventRegistryDescriptor> getClusterEventRegistryDescriptors()
	{
		return clusterEventRegistryDescriptors;
	}

}
