/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.almendra.org/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.janet.cas.node;

import java.util.List;

import javax.swing.JOptionPane;

import org.almendra.commons.util.collection.StringList;
import org.almendra.janet.cas.commands.DeregisterNodeCommand;
import org.almendra.janet.cas.scheduling.Acknowledge;



/**
 * @author Oliver
 *  
 */
public class CentralAnchor extends AbstractNodeAnchor
{
	public static final String RMI_NAME = Node.RMI_LABEL_PREFIX + "central"; //$NON-NLS-1$
	public static final String NAME = "central"; //$NON-NLS-1$

	protected CentralClusterImage clusterImage = new CentralClusterImage();
	protected List<String> nodesRequestingShutdown = new StringList();

	protected CentralAnchor()
	{
		super();
	}

	/**
	 * @param node
	 */
	public CentralAnchor(Node node)
	{
		super(node);
	}

	/**
	 * @return Returns the clusterImage.
	 */
	public CentralClusterImage getCentralClusterImage()
	{
		return clusterImage;
	}

	/**
	 * @return Returns the clusterImage.
	 */
	public AbstractClusterImage getClusterImage()
	{
		return clusterImage;
	}

	/**
	 * @param nodeName
	 */
	public void registerNode(IAgentDispatcher agentDispatcher, NodeLocation nodeLocation)
	{
		clusterImage.addNodeImage(agentDispatcher, nodeLocation);
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.node.AbstractNodeAnchor#setName(java.lang.String)
	 */
	public void setName(String nodeName)
	{
		super.setName(nodeName);
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.node.AbstractNodeAnchor#getCentralDispatcher()
	 */
	public IAgentDispatcher getCentralDispatcher()
	{
		return node.getAgentDispatcher();
	}

	/**
	 * @param string
	 */
	public boolean addShutdownRequest(String nodeName)
	{
		synchronized(nodesRequestingShutdown)
		{
			nodesRequestingShutdown.add(nodeName);
			return nodesRequestingShutdown.size() == 1;
		}
	}
	
	/**
	 * @param string
	 */
	public boolean removeShutdownRequest(String nodeName)
	{
		synchronized(nodesRequestingShutdown)
		{
			return nodesRequestingShutdown.remove(nodeName);
		}
	}
	
	public String getNextShutdownRequest()
	{
		synchronized(nodesRequestingShutdown)
		{
			if(nodesRequestingShutdown.isEmpty())
				return null;
			return nodesRequestingShutdown.get(0);
		}
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.node.AbstractNodeAnchor#disconnectRequest(org.almendra.commons.janet.cas.logger.node.Node)
	 */
	public boolean disconnectRequest(Node node)
	{
		if(!mayDisconnect())
			return false;
		
		try
		{
			DeregisterNodeCommand cmd = new DeregisterNodeCommand(node.getName(), node.getRMIName());
			Acknowledge ack = new Acknowledge();
			node.getSystemApplication().getCentralAgent().accept(cmd, ack);
			ack.acquire(Node.SHUTDOWN_TIMEOUT);
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			node.getLogger().exception(e);
		}		
		node.stop();
		return true;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.node.AbstractNodeAnchor#mayDisconnect()
	 */
	public boolean mayDisconnect()
	{
		if(!super.mayDisconnect())
			return false;
		if(getClusterImage().getForeignNodesCount() > 0)
		{
			JOptionPane.showMessageDialog(null, "Central still connected with nodes!"); //$NON-NLS-1$
			return false;
		}
		return true;
	}

}
