/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.almendra.org/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.janet.cas.node;

public class NoSuchAgentException extends Exception
{
	private static final long serialVersionUID = -803779215221476095L;
	public static final int NO_ERROR = -1;
	public static final int NODE_LEVEL = 0;
	public static final int APPLICATION_LEVEL = 1;
	public static final int CAPABILITY_LEVEL = 2;
	public static final int AGENT_LEVEL = 3;
	
	protected AgentPath agentPath = null;
	protected int errorLevel = NO_ERROR;
	protected String errorElement = null;

	/**
	 * Creates a new <code>NoSuchAgentException</code> object.
	 *  
	 */
	protected NoSuchAgentException()
	{
		super();
	}

	/**
	 * Creates a new <code>NoSuchAgentException</code> object.
	 *  
	 */
	public NoSuchAgentException(AgentPath agentPath, int errorLevel, String errorElement)
	{
		super();
		this.agentPath = agentPath;
		this.errorLevel = errorLevel;
		this.errorElement = errorElement;
	}

	/**
	 * Creates a new <code>NoSuchAgentException</code> object.
	 * 
	 * @param message
	 */
	public NoSuchAgentException(String message)
	{
		super(message);
	}

	/**
	 * Creates a new <code>NoSuchAgentException</code> object.
	 * 
	 * @param message
	 * @param cause
	 */
	public NoSuchAgentException(String message, Throwable cause)
	{
		super(message, cause);
	}

	/**
	 * Creates a new <code>NoSuchAgentException</code> object.
	 * 
	 * @param cause
	 */
	public NoSuchAgentException(Throwable cause)
	{
		super(cause);
	}

	/**
	 * @return Returns the agentPath.
	 */
	public AgentPath getAgentPath()
	{
		return agentPath;
	}

	/**
	 * @return Returns the errorLevel.
	 */
	public int getErrorLevel()
	{
		return errorLevel;
	}

	/**
	 * @return Returns the errorElement.
	 */
	public String getErrorElement()
	{
		return errorElement;
	}

}
