/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.cas.node;

import org.almendra.commons.util.StringUtility;


public class NodeName
{
	protected String workstationName = null;
	protected int nodeId = -1;
	
	/**
	 * Creates a new <code>NodeName</code> object.
	 */
	protected NodeName()
	{
		super();
	}
	
	/**
	 * Creates a new <code>NodeName</code> object.
	 */
	public NodeName(String workstationName, int nodeId)
	{
		super();
		this.workstationName = workstationName;
		this.nodeId = nodeId;
	}
	
	/**
	 * Creates a new <code>NodeName</code> object.
	 */
	public NodeName(String nodeName) throws IllegalArgumentException
	{
		super();
		init(nodeName);
	}
	/**
	 * @param nodeName
	 */
	private void init(String nodeName) throws IllegalArgumentException
	{
		String invalidArgument = "invalid node name " + nodeName; //$NON-NLS-1$
		if(!StringUtility.isValid(nodeName))
			throw new IllegalArgumentException(); //$NON-NLS-1$
		int offset = nodeName.indexOf("-"); //$NON-NLS-1$
		if(offset == -1)
			throw new IllegalArgumentException(invalidArgument);
		if(offset + 1 == nodeName.length())
			throw new IllegalArgumentException(invalidArgument);
		workstationName = nodeName.substring(0, offset);
		Integer number = StringUtility.toInteger(nodeName.substring(offset + 1, nodeName.length()));
		if(number == null)
			throw new IllegalArgumentException(invalidArgument);
		nodeId = number.intValue();
	}

	/**
	 * @return Returns the nodeId.
	 */
	public int getNodeId()
	{
		return nodeId;
	}
	
	/**
	 * @return Returns the nodeId.
	 */
	public Integer getNodeIdInt()
	{
		return new Integer(nodeId);
	}
	
	/**
	 * @return Returns the workstationName.
	 */
	public String getWorkstationName()
	{
		return workstationName;
	}
}
