/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.node.gui;

import java.util.HashMap;
import java.util.Map;

import javax.swing.DefaultListModel;

import org.almendra.commons.events.ICallbackHandler;


/**
 * @author Oliver Plohmann
 *
 */
public class NodeViewsRegistry
{
	public static final String LABEL = "NodeViewsRegistry"; //$NON-NLS-1$
	
	public static final String ViewLocation = "ViewLocation"; //$NON-NLS-1$
	public static final String ViewSize = "ViewSize"; //$NON-NLS-1$
	
	protected NodeViewController controller = null;
	protected NodeView view = null;
	protected Map<String, ICallbackHandler> viewOpenHandlersByViewLabel = new HashMap<String, ICallbackHandler>();
	
	/**
	 * Creates a new <code>NodeViewsRegistry</code> object.
	 */
	protected NodeViewsRegistry()
	{
		super();
	}

	/**
	  * Creates a new <code>NodeViewsRegistry</code> object.
	  */
	public NodeViewsRegistry(NodeViewController controller, NodeView view)
	{
		super();
		this.controller = controller;
		this.view = view;
	}
	
	public boolean addViewEntry(String viewLabel, ICallbackHandler signaledOnOpen)
	{
		synchronized(viewOpenHandlersByViewLabel)
		{
			if(viewOpenHandlersByViewLabel.containsKey(viewLabel))
				return false;
			viewOpenHandlersByViewLabel.put(viewLabel, signaledOnOpen);
			DefaultListModel model = (DefaultListModel) view.getJListViews().getModel();
			model.addElement(viewLabel);
			return true;
		}
	}
	
	public boolean open(String viewLabel)
	{
		synchronized(viewOpenHandlersByViewLabel)
		{
			ICallbackHandler handler = viewOpenHandlersByViewLabel.get(viewLabel);
			if(handler == null)
				return false;
			Map<String, Object> params = new HashMap<String, Object>();
			params.put(NodeViewsRegistry.ViewLocation, view.getLocation());
			params.put(NodeViewsRegistry.ViewSize, view.getSize());
			handler.handle(params);
			return true;
		}
	}
}
