/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.almendra.org/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.janet.cas.node.interpreters;

import java.rmi.RemoteException;
import java.util.List;

import org.almendra.commons.util.exception.InsufficientPrivilegeException;
import org.almendra.janet.cas.agents.IAgentProxy;
import org.almendra.janet.cas.commands.RegisterApplicationCommand;
import org.almendra.janet.cas.node.AgentPath;
import org.almendra.janet.cas.node.IllegalAgentAccessException;
import org.almendra.janet.cas.node.InvalidCommandException;
import org.almendra.janet.cas.node.NoSuchAgentException;
import org.almendra.janet.cas.node.Node;
import org.almendra.janet.cas.node.NodeLocation;
import org.almendra.janet.cas.scheduling.CommandAccessor;


/**
 * @author Oliver
 *  
 */
public class RegisterApplicationTrait
{

	/**
	 * Creates a new <code>RegisterApplicationTrait</code> object.
	 */
	public RegisterApplicationTrait()
	{
		super();
	}

	/**
	 * @param nodeLocation
	 * @param cmdAccessor
	 * @param isRequiredResponse
	 */
	public void foreignNodeRegisterMyApplications(String foreignNodeName, NodeLocation nodeLocation, CommandAccessor cmdAccessor) 
	throws NoSuchAgentException, IllegalAgentAccessException, InsufficientPrivilegeException, RemoteException, InvalidCommandException
	{
		Node thisNode = cmdAccessor.getNodeIfPrivilege();
		List<AgentPath> agentPaths = thisNode.getApplicationRegistry().getAllAgentPaths();
		RegisterApplicationCommand command = new RegisterApplicationCommand(cmdAccessor.getNodeName(), agentPaths, nodeLocation);
		IAgentProxy agent = cmdAccessor.getSystemAgent(foreignNodeName);
		agent.accept(command);
	}

}
