/*
 * Created on 18.09.2003
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.almendra.org/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.janet.cas.schedulers;

import org.almendra.commons.threads.IExecutable;
import org.almendra.janet.cas.node.AbstractApplication;
import org.almendra.janet.cas.node.Capability;
import org.almendra.janet.cas.node.Logger;
import org.almendra.janet.cas.node.SchedulerAnchor;
import org.almendra.janet.cas.scheduling.ICommandEnvelope;


/**
 * @author Oliver Plohmann
 */
public class ApplicationScheduler<CommandEnvelopeType extends ICommandEnvelope> extends AbstractApplicationScheduler<CommandEnvelopeType> implements IExecutable
{
	public static final String LOGGER_OBJECT_NAME = "APP_SCHEDULER"; //$NON-NLS-1$

	protected ApplicationScheduler()
	{
		super();
	}

	/**
	 * @param node
	 * @param arbitrator
	 * @param verbose
	 * @param numPriorities
	 */
	public ApplicationScheduler(
		Arbitrator arbitrator,
		AbstractApplication application,
		Capability capability, 
		Logger logger,
		boolean verbose,
		int numPriorities)
	{
		super(arbitrator, application, capability, logger, verbose, numPriorities);
	}

	/**
	 * @return
	 */
	public int getNumberOfSchedulersInSameApplication()
	{
		return application.getNumberOfSchedulers();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.schedulers.ArbitratedScheduler#getLoggerObjectName()
	 */
	protected String getLoggerObjectName()
	{
		return LOGGER_OBJECT_NAME;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.schedulers.ArbitratedScheduler#getMaxPriority()
	 */
	public int getMaxPriority()
	{
		return SchedulerAnchor.APPLICATION_PRIORITY;
	}

}
