/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.scheduling;

import java.util.List;

import org.almendra.commons.IHistorySummary;
import org.almendra.commons.util.AssertUtility;


/**
 * @author Oliver
 *
 */
public class CommandHistorySummary implements IHistorySummary<CommandHistoryItem>
{
	protected long travelTime = 0;
	protected long executionTime = 0;
	protected long systemStayTime = 0;

	/**
	 * Creates a new <code>CommandHistorySummary</code> object.
	 */
	public CommandHistorySummary()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.IHistorySummary#create(java.util.List)
	 */
	public void create(List<CommandHistoryItem> historyItems)
	{
		for(CommandHistoryItem item : historyItems)
		{
			systemStayTime += item.getSendTime().getTimeInMillis() - item.getCreationTime().getTimeInMillis();
			
			// can be null if CommandResponseEnvelope
			if(item.getStartExecutionTime() != null)
				systemStayTime += item.getStartExecutionTime().getTimeInMillis() - item.getArrivalTime().getTimeInMillis();
			
			travelTime += item.getArrivalTime().getTimeInMillis() - item.getSendTime().getTimeInMillis();
			
			// can be null if CommandResponseEnvelope
			if(item.getEndExecutionTime() != null && item.getStartExecutionTime() != null)
				executionTime += item.getEndExecutionTime().getTimeInMillis() - item.getStartExecutionTime().getTimeInMillis();
		}
	}

	/**
	 * @return Returns the executionTime.
	 */
	public long getExecutionTime()
	{
		return executionTime;
	}

	/**
	 * @return Returns the systemStayTime.
	 */
	public long getSystemStayTime()
	{
		return systemStayTime;
	}

	/**
	 * @return Returns the travelTime.
	 */
	public long getTravelTime()
	{
		return travelTime;
	}
	
	public long getTotalTime()
	{
		return executionTime + systemStayTime + travelTime;
	}

	/**
	 * @param summary
	 */
	public void add(CommandHistorySummary summary)
	{
		AssertUtility.assertTrue(executionTime >= 0);
		AssertUtility.assertTrue(summary.getExecutionTime() >= 0);		
		executionTime += summary.getExecutionTime();
		
		AssertUtility.assertTrue(systemStayTime >= 0);
		AssertUtility.assertTrue(summary.getSystemStayTime() >= 0);
		systemStayTime += summary.getSystemStayTime();
		
		AssertUtility.assertTrue(travelTime >= 0);
		AssertUtility.assertTrue(summary.getTravelTime() >= 0);
		travelTime += summary.getTravelTime();
	}

}
